/*
** Module   :PARSER.H
** Abstract :Syntax hiliting parsers
**
** Copyright (C) Sergey I. Yevtushenko
**
** Log: Tue  11/03/1997   	Created
*/

#include <dict.h>

#ifndef  __PARSER_H
#define  __PARSER_H

class Parser
{
        static char def_tbl[256];
        static int tbl_ready;

    public:
        static Dictionary* pKeywords;
        int tok_len;
        int color;
        int in_comment;
        int in_quote;
        int preserve_case;
        int mask;
        char *tok;
        char *old_tok;
        char *cvt_tbl;

        Parser();

        virtual void reset(char *tok, int comment, int quote);

        virtual int next_token();
        int is_kwd();
        void SetXlat(char *t)   { cvt_tbl = t;}
};

class Parser_CPP:public Parser
{
    public:
        Parser_CPP() { preserve_case = 1; mask = MASK_CPP;}
        virtual int next_token();
};

class Parser_REXX:public Parser
{
    public:
        Parser_REXX() { mask = MASK_REXX;}
        virtual int next_token();
};

class Parser_PAS:public Parser
{
        int cmt_char;
    public:
        Parser_PAS() { mask = MASK_PAS; cmt_char = 0;}
        virtual int next_token();
};

class Parser_ASM:public Parser
{
    public:
        Parser_ASM() { mask = MASK_ASM;}
        virtual int next_token();
};

class Parser_MAKE:public Parser
{
    public:
        Parser_MAKE() { mask = MASK_MAKE;}
        virtual int next_token();
};

class Parser_HTML:public Parser
{
    public:
        Parser_HTML() { mask = MASK_HTML;}
        virtual int next_token();
};

class Parser_MAIL:public Parser
{
        char *line_start;
    public:
        Parser_MAIL():line_start(0) { mask = MASK_MAIL;}
        virtual int next_token();
        virtual void reset(char *tok, int comment, int quote);
};

#endif //__PARSER_H

