/*
** Module   :BOXCOLL.CPP
** Abstract :Edit box collection
**
** Copyright (C) Sergey I. Yevtushenko
**
** Log: Mon  16/06/1997 	Created
*/

#include <boxcoll.h>
#include <_ctype.h>
#include <vio.h>
#include <fio.h>
#include <dialog.h>
#include <stddlg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <keynames.h>
#include <version.h>
#include <rx.h>

#define INCL_DOS
#include <os2.h>

#define MK_CLR(clr)     (app_pal[CL_APPLICATION_START+(clr)])

static void APIENTRY thTimer(ULONG data)
{
    EditBoxCollection* pEdit = (EditBoxCollection*) data;
    int i = 0;

    if(!pEdit)
        return;

    while(!pEdit->isDown())
    {
        DosSleep(100);

        i++;

        if(i == 10)
        {
            pEdit->SendKey("kbTimer");
            pEdit->draw();
            i = 0;
        }
    }
}

Initor::Initor()
{
    vio_init();
    InitREXX();
}

EditBoxCollection::EditBoxCollection():Collection(10,2), init(), keys(), prof_keys()
{
    cur_box   = 0;
    shutdown  = 0;
    recording = 0;

    head = tail = 0;
    tTID = -1;

    DosCreateMutexSem(0, &hMtx, 0, 0);
    DosCreateThread(&tTID, thTimer, (ULONG)this, 0, 0x80000);

    Clipboard = new Buffer;
    Clipboard->add_line(new Line);

    load_profile(0);    //Global profile
    load_profile(1);    //Local profile

    if(!keys.Count())
    {
        shutdown = 1;
    }

    open();

#ifdef TRACELOG
    /* Extensive dump of editor data */
    printf("Editor internal data.\n");
    int i;
    printf("keys = %d\n", keys.Count());

    for(i = 0; i < keys.Count(); i++)
    {
        keydef_pair* pKey = (keydef_pair*)keys.Get(i);
        printf("\tKey %s defined\n", pKey->key);
    }

    printf("\nprofile keys = %d\n", prof_keys.Count());
    for(i = 0; i < prof_keys.Count(); i++)
    {
        Pair* pKey = (Pair*)prof_keys.Get(i);
        printf("\tKey %s defined\n", pKey->key);
    }

    printf("\nTolower table\n", prof_keys.Count());
    for(i = 0; i < 256; i++)
    {
        if(i < ' ')
            printf("\Char = '', code = 0x%02X\n",
                   tolower_cvt_table[i]);
        else
            printf("\Char = '%c', code = 0x%02X\n",
                   tolower_cvt_table[i],
                   tolower_cvt_table[i]);
    }
    printf("\nToupper table\n", prof_keys.Count());
    for(i = 0; i < 256; i++)
    {
        if(i < ' ')
            printf("\Char = '', code = 0x%02X\n",
                   toupper_cvt_table[i]);
        else
            printf("\Char = '%c', code = 0x%02X\n",
                   toupper_cvt_table[i],
                   toupper_cvt_table[i]);
    }

    printf("\nCollate table\n", prof_keys.Count());
    for(i = 0; i < 256; i++)
    {
        if(i < ' ')
            printf("\Char = '', code = 0x%02X\n",
                   collate_cvt_table[i]);
        else
            printf("\Char = '%c', code = 0x%02X\n",
                   collate_cvt_table[i],
                   collate_cvt_table[i]);
    }
    printf("\nData dump complete.\n");
#endif

}

void EditBoxCollection::Done()
{
    DosWaitThread(&tTID, DCWW_WAIT);
    RemoveAll();
    vio_shutdown();
    DosCloseMutexSem(hMtx);

#ifdef PRINT_HEAP_STATS
    void print_heap_stat(void);
    print_heap_stat();
#endif
}

void EditBoxCollection::Free(Ptr p)
{
    delete (EditBox*)p;
}

void EditBoxCollection::SendKey(char *key)
{
    KeyInfo k;

    k.skey = shIsCtrl;
    strcpy(k.KeyName, key);
    Dispatcher(k);
}

EditBox* EditBoxCollection::current()
{
    return GetBox(cur_box);
}

EditBox* EditBoxCollection::next()
{
    cur_box++;
    if(cur_box >= Count())
        cur_box = 0;
    return current();
}

EditBox* EditBoxCollection::prev()
{
    cur_box--;
    if(cur_box < 0)
        cur_box = Count()-1;
    return current();
}

EditBox* EditBoxCollection::select(int new_cur)
{
    if(new_cur < Count())
        cur_box = new_cur;
    return current();
}

EditBox* EditBoxCollection::locate(int num)
{
    for(int i = 0; i < Count(); i++)
    {
        if(GetBox(i)->number() == num)
        {
            select(i);
            break;
        }
    }
    return current();
}

void EditBoxCollection::select(EditBox* new_box)
{
    for(int i = 0; i < Count(); i++)
    {
        if(new_box == GetBox(i))
        {
            select(i);
            break;
        }
    }
}

EditBox* EditBoxCollection::open()
{
    EditBox* new_box = new EditBox(0,0, Rows - 1, Cols);

    if(iUpperStatus)
        new_box->row++;

    Add(new_box);
    return new_box;
}

void EditBoxCollection::close()
{
    EditBox* box = (EditBox *)Remove(cur_box);
    delete box;

    if(!Count())
        open();
}

int EditBoxCollection::opened(char *fname)
{
    char _cname[FED_MAXPATH];

    _fullpath(_cname, fname, sizeof(_cname));

    for(int i = 0; i < Count(); i++)
    {
        if(!__cstrcmp(GetBox(i)->get_name(), _cname))
            return i;
    }
    return -1;
}

void EditBoxCollection::draw()
{
    if(!current())
        return;

    current()->draw();

    char* str = statusline;
    char* ptr = cName;
    int fminlen = 0;

    for(;*str; str++)
    {
        if(*str == '%')
        {
            char *out = 0;
            int minlen = 0;
            str++;

            if(!*str)
                break;

            while(__isdd(*str))
            {
                minlen *= 10;
                minlen += *str++ - '0';
            }

            switch(*str)
            {
                case 'n': out = cvt_num(current()->number(), 0);        	break;
                case 'r': out = cvt_num(current()->get_edit_row(), minlen); break;
                case 'c': out = cvt_num(current()->get_edit_col(), minlen); break;
                case 'd': out = cvt_num(current()->get_cur_char(), minlen); break;
                case 'p': out = cvt_num(current()->get_rel_pos() , minlen); break;
                case 'x': out = xcvt_char(current()->get_cur_char());   	break;
                case 'h': out = hi_names[current()->get_hiliting()];    	break;
                case 'l': out = current()->get_cur_cp();                    break;

                case 'a': *ptr++ = (current()->get_auto_indent() ? 'I':' '); break;
                case 'u': *ptr++ = (current()->get_changed() ? '*':' ');     break;
                case 't': *ptr++ = (current()->get_unix() ? 'U':'D');        break;

                case 'f': out = current()->get_fmt_name(minlen, '');
                          fminlen = minlen;
                          break;

                default:
                    if(*str)
                        *ptr++ = *str;
                    break;
            }

            if(out)
                while(*out)
                    *ptr++ = *out++;
            continue;
        }
        *ptr++ = *str;
    }
    *ptr = 0;

    int sRow = (iUpperStatus) ? (0):(Rows - 1);

    vio_print2(sRow, 0, cName, Cols, MK_CLR(CL_STATUSLINE));

    if(recording)
        vio_print2(sRow, Cols - 1, "R", 1, 0xF0);

    strcpy(cName, "FED-");
    strcat(cName, current()->get_fmt_name(fminlen));
    set_title(cName);
}

int EditBoxCollection::check_save()
{
    if(!current())
        return 0;

    EditBox* cur = current();

    do
    {
        if(current()->get_changed())
        {
            draw();

            int rc = AChoice(5, 5, Yes_No, "Save?");

            switch(rc)
            {
                case -1:
                case  2:
                    select(cur);
                    return -1;
                case  0:

                    if(current()->is_untitled())
                    {
                        rc = FileDialog(5, 5, cName, 2);
                        if(!rc)
                            current()->save_as(cName);
                        else
                            return -1;
                    }
                    if(current()->save())
                    {
                        MessageBox("Error saving file!");
                        return -1;
                    }
            }
        }
        else
            current()->save(); //Indeed just put EA and ignore error

        next();
    }
    while(current() != cur);
    return 0;
}

void EditBoxCollection::search_proc()
{
    if(!current())
        return;

    int rc = current()->search(Flags, Search);

    if(!rc)
    {
        if(iVSearch)
            MessageBox("\n No occurences found! \n");
        return;
    }

    if(rc &&
       (strchr(Flags, 'r') ||
        strchr(Flags, 'R')))
    {   // Replace mode
        if((strchr(Flags, 'n') ||
            strchr(Flags, 'N'))) //Noask
        {
            do
            {
                current()->replace(Replace);
            }
            while(current()->search(Flags, Search));

            if(iVSearch)
                MessageBox("\n Done. \n");
        }
        else //Confirmation
        {
            do
            {
                int rr;
                int cc;

                if(current()->get_cur_row() < (Rows - 7))
                    rr = current()->get_cur_row() + 2;
                else
                    rr = current()->get_cur_row() - 7;

                if(current()->get_cur_col() < (Cols - 12))
                    cc = current()->get_cur_col();
                else
                    cc = current()->get_cur_col() - 12;

                draw();

                rc = AChoice(rr, cc, Yes_No, "Replace?");
                if(rc == 0)
                    current()->replace(Replace);
                if(rc == 2 || rc == -1)
                    break;
            }
            while(current()->search(Flags, Search));

            if(rc != 2 && rc != -1 && iVSearch)
                MessageBox("\n Done. \n");
        }
    }
}

void EditBoxCollection::track_beg()
{
    for(int i = 0; i < Count(); i++)
    {
        GetBox(i)->track_beg();
    }
}

void EditBoxCollection::track_end()
{
    for(int i = 0; i < Count(); i++)
    {
        GetBox(i)->track_end();
    }
}

void EditBoxCollection::track_cancel()
{
    for(int i = 0; i < Count(); i++)
    {
        GetBox(i)->track_cancel();
    }
}

void EditBoxCollection::usual_key(KeyInfo& k)
{
    if(current()->get_mark_state())
        current()->clear(*current());

    if(current()->get_ins_mode())
        current()->ins_char(*current(), k.key);
    else
        current()->replace_char(*current(), k.key);
}


