/*
** Module   :BRACKET.CPP
** Abstract :Bracket matching routine
**
** Copyright (C) Sergey I. Yevtushenko
**
** Log: Mon  15/03/1998     Created
*/

#include <buffer.h>
#include <stdlib.h>
#include <string.h>
#include <_ctype.h>
#include <parser.h>
#include <_search.h>
#include <version.h>

#define UNDO    1

#ifndef max
#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif


//-----------------------------------------
// Bracket matching v 0.0.1a
//-----------------------------------------

struct brackets_pair
{
    char cLB;
    char cRB;
};

static brackets_pair brackets[]=
{
    {'(',')'},
    {'[',']'},
    {'{','}'},
    {'<','>'}
};

#define BM_FORWARD  0
#define BM_BACKWARD 1
#define BM_NONE     2

void Buffer::check_pos(int _row, int _col, int& comment, int& quote)
{
    PLine ln = line(_row);
    comment = quote = 0;

    if(ln)
    {
        Parser * parser = gen_parser();
        char *tmp = new char[(ln->len() + 1)];

        parser->reset(tmp, comment, quote);

        while(*parser->tok)
        {
            parser->tok += parser->next_token();

            if((parser->tok - tmp) >= _col)
            {
            	comment = parser->in_comment;
                quote   = parser->in_quote;
                break;
            }
        }

        if(ln->len() < _col)
        {
	        comment = parser->in_comment;
    	    quote   = parser->in_quote;
        }
        delete tmp;
        delete parser;
    }
}

void Buffer::match_bracket(Rect& rect)
{
    int i;
    int direction = BM_NONE;
    int  pair  = 0;
    int  level = 0;

    char bracket = (char)abs_line()->char_at(abs_col());

    for(i = 0; i < (sizeof(brackets)/sizeof(brackets[0])); i++)
    {
        if(bracket == brackets[i].cLB)
        {
            // Bracket found, direction forward
            direction = BM_FORWARD;
            pair = i;
            break;
        }

        if(bracket == brackets[i].cRB)
        {
            // Bracket found, direction backward
            direction = BM_BACKWARD;
            pair = i;
            break;
        }
    }

    if(direction == BM_NONE)
        return;

    // Look for token for current point
    // This token can be inside comment or quote
    // if we found such case, lookup for matching bracket will
    // be limited for comment or quote

    int comment = abs_line()->in_comment;
    int quote   = abs_line()->in_quote;

    check_pos(abs_row(), abs_col(), comment, quote);

    if(direction == BM_FORWARD)
    {
        //Direction: forward
        //quickly looking for bracket, then check other parameters
        int _row = abs_row();
        int _col = abs_col()+1;

        while(_row < Count())
        {
            PLine ln = line(_row);

            for(; _col < ln->len(); _col++)
            {
                if(ln->char_at(_col) == brackets[pair].cLB)
                {
                    level++;
                    continue;
                }

                if(ln->char_at(_col) == brackets[pair].cRB)
                {
                    if(level)
                    {
                        level--;
                        continue;
                    }

                    if(_row != abs_row())
                        goto_line(rect, _row);
                    goto_col(rect, _col);
                    return;
                }
            }
            _row++;
            _col = 0;
        }
    }
    else
    {
        int _row = abs_row();
        int _col = abs_col()-1;

        while(_row >= 0)
        {
            PLine ln = line(_row);

            for(; _col >= 0; _col--)
            {
                if(ln->char_at(_col) == brackets[pair].cRB)
                {
                    level++;
                    continue;
                }

                if(ln->char_at(_col) == brackets[pair].cLB)
                {
                    if(level)
                    {
                        level--;
                        continue;
                    }

                    if(_row != abs_row())
                        goto_line(rect, _row);
                    goto_col(rect, _col);
                    return;
                }
            }
            _row--;
            _col = line(_row) ? line(_row)->len() : 0;
        }
    }
}

