/*
** Module   :HEAP.CPP
** Abstract :Heap manager for FED
**
** Copyright (C) Sergey I. Yevtushenko
**
** Log: Wed  12/11/1997   	Moved from FED.CPP
*/

#include <stdlib.h>
#include <os2.h>
#include <version.h>

#ifdef __IBMCPP__
#define __FED_ALLOC__
#endif

#ifndef __FED_ALLOC__

#define MEM_TRESHOLD    16

#if defined(__WATCOMC__)
#undef  __FED_DEBUG__
#endif

#ifndef __FED_DEBUG__

struct __mem_chunk
{
    union
    {
        unsigned size;
        struct __mem_chunk * next;
    }hdr;
};

typedef struct __mem_chunk *pchunk;

#define NUM_CHUNKS  32
static pchunk chunks[NUM_CHUNKS];

void splitchunk(unsigned startpos)
{
    if(!chunks[startpos+1])
    {
        if(startpos >= MEM_TRESHOLD)
        {
            APIRET rc = DosAllocMem((PPVOID)&chunks[startpos+1],
                                    (1 << (startpos + 1))+1,
                                    0
                                    | PAG_READ
                                    | PAG_WRITE
                                    | PAG_COMMIT);
            chunks[startpos+1]->hdr.next = 0;
        }
        else
            splitchunk(startpos+1);
    }

    {
        pchunk s0;
        pchunk s1;
        unsigned size = 1 << (startpos);
        pchunk p = chunks[startpos+1];

        chunks[startpos+1] = p->hdr.next;
        s0 = p;
        s1 = (pchunk)(((unsigned char *)p)+size);
        s0->hdr.next = s1;
        s1->hdr.next = chunks[startpos];
        chunks[startpos] = s0;
    }
}
#ifdef __GNUC__
void *malloc (size_t size)
#else
void *alloc_mem (size_t size)
#endif
{
    unsigned pos = 0;
    unsigned bit = 1;
    pchunk p;

    if(!size)
        return 0;

    size += 4;

    do
    {
        pos++;
        bit <<= 1;
    }
    while(bit < size);

    if(!chunks[pos])
    {
        splitchunk(pos);
    }
    p = chunks[pos];
    chunks[pos] = p->hdr.next;
    p->hdr.size = bit;
    return ((unsigned char *)p)+4;
}
#ifdef __GNUC__
void free (void *ptr)
#else
void free_mem (void *ptr)
#endif
{
    unsigned pos = 0;
    unsigned bit = 1;
    pchunk p;
    unsigned size;

    if(!ptr)
        return;

    p = (pchunk)(((unsigned char *)ptr)-4);
    size = p->hdr.size;

    do
    {
        pos++;
        bit <<= 1;
    }
    while(bit < size);

    p->hdr.next = chunks[pos];
    chunks[pos] = p;
}


#ifndef __GNUC__
void *operator new( size_t sz)
{
    return alloc_mem(sz);
}
#if defined(__WATCOMC__) || defined(__IBMCPP__)
void operator delete( void * p)
{
    free_mem(p);
}
#else
void operator delete(void *p, const char*, unsigned)
{
    free_mem((void *)p);
}
#endif
#endif

#ifdef PRINT_HEAP_STATS
#include <stdio.h>

void print_heap_stat(void)
{
    for(int i = 0; i < NUM_CHUNKS; i++)
    {
        int    j;
        pchunk p;
        for(j = 0, p = chunks[i]; p ;j++)
            p = p->hdr.next;
        printf("Row %d(%d) - %d\n", i, 1 << i, j);
    }
}

#endif
#else
#endif

#else
int __make_librarian_happy__;
#endif


