/*
** Module   :PARSER.CPP
** Abstract :Sintax hiliting language-specific parsers
**
** Copyright (C) Sergey I. Yevtushenko
**
** Log: Tue  11/03/1997   	Created
**      Sat  12/04/1997   	Added REXX parser
*/

#include <common.h>
#include <parser.h>
#include <string.h>
#include <_ctype.h>
#include <version.h>

//----------------------------------------------------------------------
//
// Class Parser
//
//----------------------------------------------------------------------
struct kwdPair
{
    char *key;
    int mask;
};

typedef kwdPair* Pkwd;

kwdPair keywords [] =
{
    { "absolute"      , MASK_PAS },
    { "abstract"      , MASK_JAVA},
    { "address"       , MASK_REXX},
    { "all"           , MASK_REXX},
    { "and"           , MASK_PAS },
    { "arg"           , MASK_REXX},
    { "array"         , MASK_PAS },
    { "asm"           , MASK_PAS },
    { "assembler"     , MASK_PAS },
    { "auto"          , MASK_CPP },
    { "begin"         , MASK_PAS },
    { "bool"          , MASK_CPP },
    { "boolean"       , MASK_JAVA},
    { "break"         , MASK_CPP | MASK_JAVA},
    { "by"            , MASK_REXX},
    { "byte"          , MASK_ASM | MASK_JAVA},
    { "call"          , MASK_ASM | MASK_REXX},
    { "case"          , MASK_CPP | MASK_PAS | MASK_JAVA},
    { "catch"         , MASK_CPP | MASK_JAVA},
    { "char"          , MASK_CPP },
    { "class"         , MASK_CPP | MASK_JAVA},
    { "commands"      , MASK_REXX},
    { "const"         , MASK_CPP | MASK_PAS | MASK_JAVA},
    { "constructor"   , MASK_PAS },
    { "continue"      , MASK_CPP | MASK_JAVA},
    { "default"       , MASK_CPP | MASK_JAVA},
    { "define"        , MASK_CPP },
    { "delete"        , MASK_CPP },
    { "destructor"    , MASK_PAS },
    { "digits"        , MASK_REXX},
    { "div"           , MASK_PAS },
    { "do"            , MASK_REXX | MASK_CPP | MASK_PAS | MASK_JAVA},
    { "double"        , MASK_CPP | MASK_JAVA},
    { "downto"        , MASK_PAS },
    { "drop"          , MASK_REXX},
    { "dword"         , MASK_ASM },
    { "else"          , MASK_REXX | MASK_CPP | MASK_PAS | MASK_JAVA},
    { "end"           , MASK_REXX | MASK_ASM | MASK_PAS},
    { "endif"         , MASK_CPP },
    { "endp"          , MASK_ASM },
    { "ends"          , MASK_ASM },
    { "engineering"   , MASK_REXX},
    { "entry"         , MASK_CPP },
    { "enum"          , MASK_CPP },
    { "error"         , MASK_REXX},
    { "errors"        , MASK_REXX},
    { "exit"          , MASK_REXX},
    { "expose"        , MASK_REXX},
    { "extends"       , MASK_JAVA},
    { "extern"        , MASK_CPP },
    { "external"      , MASK_PAS },
    { "false"         , MASK_CPP | MASK_JAVA},
    { "far"           , MASK_PAS },
    { "file"          , MASK_PAS },
    { "final"         , MASK_JAVA},
    { "finally"       , MASK_JAVA},
    { "float"         , MASK_CPP | MASK_JAVA},
    { "for"           , MASK_REXX | MASK_CPP | MASK_PAS | MASK_JAVA},
    { "forever"       , MASK_REXX},
    { "form"          , MASK_REXX},
    { "forward"       , MASK_PAS },
    { "friend"        , MASK_CPP },
    { "function"      , MASK_PAS },
    { "fuzz"          , MASK_REXX},
    { "goto"          , MASK_CPP | MASK_PAS | MASK_JAVA},
    { "halt"          , MASK_REXX},
    { "if"            , MASK_REXX | MASK_CPP | MASK_PAS | MASK_JAVA},
    { "ifdef"         , MASK_CPP },
    { "ifndef"        , MASK_CPP },
    { "implementation", MASK_PAS },
    { "implements"    , MASK_JAVA},
    { "import"        , MASK_JAVA},
    { "in"            , MASK_PAS },
    { "include"       , MASK_CPP },
    { "inherited"     , MASK_PAS },
    { "inline"        , MASK_CPP | MASK_PAS},
    { "instanceof"    , MASK_JAVA},
    { "int"           , MASK_CPP | MASK_JAVA},
    { "interface"     , MASK_PAS | MASK_JAVA},
    { "intermediates" , MASK_REXX},
    { "interpret"     , MASK_REXX},
    { "interrupt"     , MASK_PAS },
    { "iterate"       , MASK_REXX},
    { "label"         , MASK_ASM | MASK_PAS},
    { "labels"        , MASK_REXX},
    { "leave"         , MASK_REXX},
    { "long"          , MASK_CPP | MASK_JAVA},
    { "mod"           , MASK_PAS },
    { "native"        , MASK_JAVA},
    { "near"          , MASK_ASM | MASK_PAS},
    { "negative"      , MASK_REXX},
    { "new"           , MASK_CPP | MASK_JAVA},
    { "nil"           , MASK_PAS },
    { "nop"           , MASK_REXX},
    { "normal"        , MASK_REXX},
    { "not"           , MASK_PAS },
    { "novalue"       , MASK_REXX},
    { "null"          , MASK_JAVA},
    { "numeric"       , MASK_REXX},
    { "object"        , MASK_PAS },
    { "of"            , MASK_PAS },
    { "off"           , MASK_REXX},
    { "offset"        , MASK_ASM },
    { "on"            , MASK_REXX},
    { "or"            , MASK_PAS },
    { "otherwise"     , MASK_REXX},
    { "package"       , MASK_JAVA},
    { "packed"        , MASK_PAS },
    { "parse"         , MASK_REXX},
    { "private"       , MASK_CPP | MASK_PAS | MASK_JAVA},
    { "proc"          , MASK_ASM },
    { "procedure"     , MASK_REXX | MASK_PAS},
    { "program"       , MASK_PAS },
    { "protected"     , MASK_CPP | MASK_JAVA},
    { "ptr"           , MASK_ASM },
    { "public"        , MASK_CPP | MASK_PAS | MASK_JAVA},
    { "pull"          , MASK_REXX},
    { "push"          , MASK_REXX},
    { "queue"         , MASK_REXX},
    { "qword"         , MASK_ASM },
    { "record"        , MASK_PAS },
    { "register"      , MASK_CPP },
    { "repeat"        , MASK_PAS },
    { "results"       , MASK_REXX},
    { "return"        , MASK_REXX | MASK_CPP | MASK_JAVA},
    { "say"           , MASK_REXX},
    { "scan"          , MASK_REXX},
    { "scientific"    , MASK_REXX},
    { "segment"       , MASK_ASM },
    { "select"        , MASK_REXX},
    { "set"           , MASK_PAS },
    { "shl"           , MASK_PAS },
    { "short"         , MASK_CPP | MASK_JAVA},
    { "shr"           , MASK_PAS },
    { "sigl"          , MASK_REXX},
    { "signal"        , MASK_REXX},
    { "signed"        , MASK_CPP },
    { "sizeof"        , MASK_CPP },
    { "static"        , MASK_CPP | MASK_JAVA},
    { "string"        , MASK_PAS },
    { "struc"         , MASK_ASM },
    { "struct"        , MASK_CPP },
    { "super"         , MASK_JAVA},
    { "switch"        , MASK_CPP | MASK_JAVA},
    { "synchronized"  , MASK_JAVA},
    { "syntax"        , MASK_REXX},
    { "then"          , MASK_REXX | MASK_PAS},
    { "this"          , MASK_CPP | MASK_JAVA},
    { "throw"         , MASK_CPP | MASK_JAVA},
    { "throws"        , MASK_JAVA},
    { "to"            , MASK_REXX | MASK_PAS},
    { "trace"         , MASK_REXX},
    { "transient"     , MASK_JAVA},
    { "true"          , MASK_CPP | MASK_JAVA},
    { "try"           , MASK_CPP | MASK_JAVA},
    { "type"          , MASK_PAS },
    { "typedef"       , MASK_CPP },
    { "union"         , MASK_CPP },
    { "unit"          , MASK_PAS },
    { "unsigned"      , MASK_CPP },
    { "until"         , MASK_REXX | MASK_PAS},
    { "upper"         , MASK_REXX},
    { "uses"          , MASK_PAS },
    { "value"         , MASK_REXX},
    { "var"           , MASK_REXX | MASK_PAS},
    { "virtual"       , MASK_CPP | MASK_PAS },
    { "void"          , MASK_CPP | MASK_JAVA},
    { "volatile"      , MASK_CPP | MASK_JAVA},
    { "when"          , MASK_REXX},
    { "while"         , MASK_REXX | MASK_CPP | MASK_PAS | MASK_JAVA},
    { "with"          , MASK_REXX | MASK_PAS},
    { "word"          , MASK_ASM },
    { "xor"           , MASK_PAS }
};

Dictionary *Parser::pKeywords = 0;

char Parser::def_tbl[256];
int Parser::tbl_ready = 0;

Parser::Parser()
{
    if(!pKeywords)
    {
        pKeywords = new Dictionary(1,0,0);
        for(int i = 0; i < (sizeof(keywords)/sizeof(keywords[0])); i++)
            pKeywords->Add(&keywords[i]);
    }
    reset(0, 0, 0);
    preserve_case = 0;
    mask = 0;

    if(!tbl_ready)
    {
        for(int j = 0; j < 256; j++)
            def_tbl[j] = j;

        tbl_ready = 1;
    }

    cvt_tbl = def_tbl;
}

void Parser::reset(char *token, int comment, int quote)
{
    tok_len = 0;
    color   = 0;
    in_comment = comment;
    in_quote   = quote;
    tok     = token;
    old_tok = token;
}

int Parser::next_token()
{
    old_tok = tok;
    tok_len = 0;
    color = CL_DEFAULT;
    char *tmp = tok;
    while(*tmp)
    {
        tmp++;
        tok_len++;
    }
    return tok_len;
}

int Parser::is_kwd()
{
    Pkwd pKwd = (Pkwd) pKeywords->IsIn(tok, tok_len, preserve_case);
    if(pKwd && (pKwd->mask & mask))
        return 1;
    return 0;
}

