/*
** Module   :PROFILE.CPP
** Abstract :FED profile (FED.INI) reader/parser
**
** Copyright (C) Sergey I. Yevtushenko
**
** Log: Wed  17/03/1998     Created
*/

#define INCL_DOS
#include <os2.h>
#include <common.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <_ctype.h>
#include <fio.h>
#include <boxcoll.h>
#include <version.h>

#define P_INT  1
#define P_CHAR 2
#define P_PSZ  3

struct stPairDef;
typedef struct stPairDef* PPairDef;

struct stPairDef
{
    char *index;
    int type;
    void *value;
};

stPairDef ProfileDef[]=
{
    {"color.app.default"            , P_CHAR, app_pal + CL_APPLICATION_START+CL_DEFAULT       },
    {"color.app.status"             , P_CHAR, app_pal + CL_APPLICATION_START+CL_STATUSLINE    },
    {"color.dialog.default"         , P_CHAR, app_pal + CL_DIALOG_START+CL_DEFAULT            },
    {"color.dialog.hilite"          , P_CHAR, app_pal + CL_DIALOG_START+CL_BORDER             },
    {"color.edit.comment"           , P_CHAR, app_pal + CL_EDITBOX_START+CL_COMMENT           },
    {"color.edit.const"             , P_CHAR, app_pal + CL_EDITBOX_START+CL_CONST             },
    {"color.edit.default"           , P_CHAR, app_pal + CL_EDITBOX_START+CL_DEFAULT           },
    {"color.edit.eof"               , P_CHAR, app_pal + CL_EDITBOX_START+CL_EOF               },
    {"color.edit.function"          , P_CHAR, app_pal + CL_EDITBOX_START+CL_FUNCTION          },
    {"color.edit.ident"             , P_CHAR, app_pal + CL_EDITBOX_START+CL_IDENT             },
    {"color.edit.number"            , P_CHAR, app_pal + CL_EDITBOX_START+CL_NUMBER            },
    {"color.edit.preproc"           , P_CHAR, app_pal + CL_EDITBOX_START+CL_PREPROC           },
    {"color.edit.selection"         , P_CHAR, app_pal + CL_EDITBOX_START+CL_SELECTION         },
    {"color.edit.semicol"           , P_CHAR, app_pal + CL_EDITBOX_START+CL_SEMICOL           },
    {"color.edit.stdword"           , P_CHAR, app_pal + CL_EDITBOX_START+CL_STDWORD           },
    {"color.edit.xnumber"           , P_CHAR, app_pal + CL_EDITBOX_START+CL_XNUMBER           },
    {"color.line.active.comment"    , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_COMMENT         },
    {"color.line.active.const"      , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_CONST           },
    {"color.line.active.default"    , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_DEFAULT         },
    {"color.line.active.eof"        , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_EOF             },
    {"color.line.active.function"   , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_FUNCTION        },
    {"color.line.active.ident"      , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_IDENT           },
    {"color.line.active.number"     , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_NUMBER          },
    {"color.line.active.preproc"    , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_PREPROC         },
    {"color.line.active.selection"  , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_SELECTION       },
    {"color.line.active.semicol"    , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_SEMICOL         },
    {"color.line.active.stdword"    , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_STDWORD         },
    {"color.line.active.xnumber"    , P_CHAR, app_pal + CL_EDITLINE_ACTIVE+CL_XNUMBER         },
    {"color.line.inactive.comment"  , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_COMMENT       },
    {"color.line.inactive.const"    , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_CONST         },
    {"color.line.inactive.default"  , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_DEFAULT       },
    {"color.line.inactive.eof"      , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_EOF           },
    {"color.line.inactive.function" , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_FUNCTION      },
    {"color.line.inactive.ident"    , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_IDENT         },
    {"color.line.inactive.number"   , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_NUMBER        },
    {"color.line.inactive.preproc"  , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_PREPROC       },
    {"color.line.inactive.selection", P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_SELECTION     },
    {"color.line.inactive.semicol"  , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_SEMICOL       },
    {"color.line.inactive.stdword"  , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_STDWORD       },
    {"color.line.inactive.xnumber"  , P_CHAR, app_pal + CL_EDITLINE_INACTIVE+CL_XNUMBER       },
    {"color.list.active.current"    , P_CHAR, app_pal + CL_LISTBOX_ACTIVE+CL_CURRENT          },
    {"color.list.active.currsel"    , P_CHAR, app_pal + CL_LISTBOX_ACTIVE+CL_CURRSEL          },
    {"color.list.active.default"    , P_CHAR, app_pal + CL_LISTBOX_ACTIVE+CL_DEFAULT          },
    {"color.list.active.selection"  , P_CHAR, app_pal + CL_LISTBOX_ACTIVE+CL_SELECTION        },
    {"color.list.inactive.current"  , P_CHAR, app_pal + CL_LISTBOX_INACTIVE+CL_CURRENT        },
    {"color.list.inactive.currsel"  , P_CHAR, app_pal + CL_LISTBOX_INACTIVE+CL_CURRSEL        },
    {"color.list.inactive.default"  , P_CHAR, app_pal + CL_LISTBOX_INACTIVE+CL_DEFAULT        },
    {"color.list.inactive.selection", P_CHAR, app_pal + CL_LISTBOX_INACTIVE+CL_SELECTION      },
    {"color.menu.active.current"    , P_CHAR, app_pal + CL_MENU+CL_CURRENT                    },
    {"color.menu.active.currsel"    , P_CHAR, app_pal + CL_MENU+CL_CURRSEL                    },
    {"color.menu.active.default"    , P_CHAR, app_pal + CL_MENU+CL_DEFAULT                    },
    {"color.menu.active.selection"  , P_CHAR, app_pal + CL_MENU+CL_SELECTION                  },
    {"color.static.default"         , P_CHAR, app_pal + CL_STEXT_START+CL_DEFAULT             },
    {"color.static.hilite"          , P_CHAR, app_pal + CL_STEXT_START+CL_HILITE              },
    {"cursor.shape.insert"          , P_INT , iShape},
    {"cursor.shape.overwrite"       , P_INT , iShape + 1},
    {"editor.ctrlbreak.action"      , P_INT , &iCtrlBrk},
    {"editor.default.format"        , P_INT , &iDefType},
//    {"editor.force.pm"              , P_INT , &iForce},
    {"editor.helptext"              , P_PSZ , &help_text},
    {"editor.hiliting"              , P_PSZ , &hi_map},
    {"editor.no.ea"                 , P_INT , &iNoEA},
    {"editor.statusline"            , P_PSZ , &statusline},
    {"editor.statuspos"             , P_INT , &iUpperStatus},
    {"editor.tabwidth"              , P_INT , &iTabWidth},
    {"editor.untitled"              , P_PSZ , &untitled},
    {"editor.verbose.search"        , P_INT , &iVSearch},
    {"editor.file.name.reduce"      , P_INT , &iFileName}
};

ProfileDictionary::ProfileDictionary():Dictionary(1, 0, 0)
{
    for(int i = 0; i < sizeof(ProfileDef)/sizeof(ProfileDef[0]); i++)
    {
        //printf("init: %s=%08x\n", ProfileDef[i].index, ProfileDef[i].value);
        Add(&ProfileDef[i]);
    }
}

int get_ini_name(char *cProgName, char *ini_ext)
{
    char* lastdot;
    char* lastslash;

    strcpy(cProgName, _cFedPATH);

    lastdot   = strrchr(cProgName, '.');
    lastslash = strrchr(cProgName, '\\');

    if(lastdot > lastslash) //dot really in extension, not in path
        strcpy(lastdot, ini_ext);
    else
        strcat(cProgName, ini_ext);

    return 0;
}

void EditBoxCollection::load_profile(int mode)
{
    char cProgName[FED_MAXPATH];

    if(!mode)
    {
        if(get_ini_name(cProgName, ".ini"))
            return;
    }
    else
    {
        strcpy(cProgName, "fed.ini");
    }

    char *orig_file = _ld_file(cProgName);

    if(orig_file)
    	load_profile(orig_file);

    _fr_file(orig_file);
}

void EditBoxCollection::load_profile(char* orig_file)
{
    char *ptr;
    char *str;
    int linecnt = 1;

    for(ptr = str = orig_file; *str;)
    {
        int merge_lines = 0;
        do
        {
        	while(*str && *str != '\n')
                str++;

            linecnt++;

            if(*str)
            {
                if((*(str-1) == '\r' && *(str-2)==','))
                {
                    *str = ' ';
                    *(str - 1) = ' ';
                    *(str - 2) = ' ';
                    merge_lines = 1;
                }
                else
                    if(*(str-1) == ',')
                    {
                        *str = ' ';
                        *(str - 1) = ' ';
                    }
                    else
                        merge_lines = 0;
            }
        }
        while(merge_lines);

        if(*str)
        {
            if(*(str - 1) == '\r')
                *(str - 1) = 0;
            *str++ = 0;
            if(*str == '\r' || *str == '\n')
                *str++ = 0;
        }

        //Line found, parse it

        while(*ptr)
        {
            int ignore;
            ignore = 0;

            if(*ptr == ';' ||
               *ptr == '#' ||
               *ptr == '/' ||
               ignore) //skip comments
                break;

            if(__isis(*ptr))
            {
                int j = 0;
                for(;__isic(ptr[j]) || ptr[j] == '.'; j++)
                    ;

                if(j > 0)
                {
                    PPairDef pProf;

                    pProf = (PPairDef) prof_keys.IsIn(ptr, j, 0);

                    if(pProf)
                    {
                        // Scan again to find new value
                        int xnum   = 0;
                        int ivalue = 0;
                        int sign   = 0;

                        ptr += j;
                        while(__issp(*ptr))
                            ptr++;

                        if(*ptr==':' ||
                           *ptr=='=')
                        {
                            ptr++;

                            while(__issp(*ptr)) //Skip whitespaces
    	                        ptr++;

                            switch(pProf->type)
                            {
                                case P_INT:
                                case P_CHAR:

                                    j = 0;

                                    if(ptr[j] == '-')
                                    {
                                        sign = 1;
                                        j++;
                                    }

                                    if(ptr[j] == '+')
                                        j++;

                                    if(j) //after sign can be spaces
                                    {
                                        while(__issp(ptr[j]) && ptr[j])
                                            j++;

                                        if(!ptr[j])
                                        {
                                            //printf("Missing value for key %s in line %d\n",
                                            //       pProf->index,
                                            //       linecnt);
                                            ignore = 1; // comment out rest
                                            continue;
                                        }
                                        ptr += j;
                                        j = 0;
                                    }

                                    while((__ishd(*ptr) ||
                                           __to_upper(*ptr) == 'X') &&
                                           !ignore)
                                    {
                                        switch(__to_upper(*ptr))
                                        {
                                            case '0': case '1':
                                            case '2': case '3':
                                            case '4': case '5':
                                            case '6': case '7':
                                            case '8': case '9':
                                            case 'A': case 'B':
                                            case 'C': case 'D':
                                            case 'E': case 'F':

                                                if(__to_upper(*ptr) > '9' && !xnum)
                                                {
                                                    //printf("Wrong digits in number in line %d\n", linecnt);
                                                    ignore = 1;
                                                    continue;
                                                }

                                                if(xnum)
                                                {
                                                    ivalue <<= 4;

                                                    if(__to_upper(*ptr) > '9')
                                                        ivalue |= __to_upper(*ptr) - 'A' + 0x0A;
                                                    else
                                                        ivalue |= *ptr - '0';
                                                }
                                                else
                                                {
                                                    ivalue *= 10;
                                                    ivalue += *ptr - '0';
                                                }
                                                break;

                                            case 'X':
                                                xnum = 1;
                                                break;
                                        }
                                        ptr++;
                                    }

                                    if(!ignore)
                                    {
                                        if(sign)
                                            ivalue = -ivalue;

                                        //printf("%s=%d, %08x\n", pProf->index, ivalue, pProf->value);

                                        if(pProf->value)
                                        {
                                        	if(pProf->type == P_CHAR)
                                            	*((char *)pProf->value) = (char)ivalue;

                                        	if(pProf->type == P_INT)
                                            	*((int *)pProf->value) = (int)ivalue;
                                        }
                                        //printf("%s=%d\n", pProf->index, value);
                                    }

                                    ignore = 1;
                                    break;

                                case P_PSZ:
                                    {
                                        int nlen   = 0;
                                        int inside = 0;
                                        char delim = 0;

                                        //First pass, calculate len
                                        for(j = 0; ptr[j]; j++)
                                        {
                                            if(inside)
                                            {
                                                if(ptr[j] == delim)
                                                {
                                                	inside = 0;
                                                    continue;
                                                }
                                                if(ptr[j] == '\\')
                                                    j++;
                                                nlen++;
                                            }
                                            if(ptr[j] == '"' || ptr[j] == '\'')
                                            {
                                                delim = ptr[j];
                                                inside = 1;
                                                continue;
                                            }
                                        }

                                        if(!nlen)
                                            break;

                                        char *out  = new char[nlen+1];
                                        out[nlen]  = 0;
                                        char *out2 = out;

                                        //Second pass, copy
                                        for(j = 0; ptr[j]; j++)
                                        {
                                            if(inside)
                                            {
                                                char chr = ptr[j];
                                                if(ptr[j] == delim)
                                                {
                                                	inside = 0;
                                                    continue;
                                                }
                                                if(ptr[j] == '\\')
                                                {
                                                    j++;
                                                    switch(__to_upper(ptr[j]))
                                                    {
                                                        case 'T': chr = '\t'; break;
                                                        case 'N': chr = '\n'; break;
                                                        default:  chr = ptr[j]; break;
                                                    }
                                                }
                                                *out2++ = chr;
                                            }
                                            if(ptr[j] == '"' || ptr[j] == '\'')
                                            {
                                                delim = ptr[j];
                                                inside = 1;
                                                continue;
                                            }
                                        }
                                        *(char **)(pProf->value) = out;
                                    }
                                    break;
                            }
                        }
                        else
                        {
                            //printf("Missing value for key %s in line %d\n",
                            //       pProf->index,
                            //       linecnt);
                        }
                    }
                    else
                    {
                        //This can be kedef.
                        //keydef should begin with 'kb'
                        if(__to_upper(ptr[0]) == 'K' &&
                           __to_upper(ptr[1]) == 'B' &&
                           j < KEY_NAME_LEN)
                        {
                            char *tmp = ptr + j;

                            while(__issp(*tmp))
                                tmp++;

                            if(*tmp == '=' ||
                               *tmp == ':')
                            {
                                tmp++;

                                while(__issp(*tmp))
                                    tmp++;

                                //Check for curly braces, here can be rexx
                                if(*tmp == '{')
                                {
                                    //Merge lines until next
                                    //curly brace
                                    //They should be at end of line
                                    tmp = str - 2;

                                    while(!*tmp)
                                        tmp--;

                                    if(*tmp != '}')
                                    {
                                        tmp++;
                                        if(!*tmp && !*(tmp + 1))
                                        {
                                            *tmp++ = '\r';
                                            *tmp   = '\n';
                                        }

                                        if(!*tmp)
                                            *tmp = '\n';
                                    }

                                    while(*tmp != '}' && *str)
                                    {
                                        //Find next line
                                        while(*str && *str != '\n')
                                            str++;

                                        linecnt++;

                                        if(*str)
                                        {
                                            tmp = str;

                                            while(*tmp == '\n' ||
                                                  *tmp == '\r' ||
                                                  *tmp == ' ')
                                                tmp--;

                                            if(*tmp == '}') //End
                                            {
                                                str = tmp+1;
                                                while(*str == '\n' ||
                                                      *str == '\r')
                                                    *str++ = 0;
                                            }
                                            else
                                            	str++;

                                        }
                                        else
                                            break;
                                    }

                                }

                                keys.InsKey(ptr, j);
                            }
                            else
                            {
                                //printf("Missing value for key definition in line %d\n",
                                //       linecnt);
                            }

                        }
                        //else
                        //error message
                        //printf("Invalid key %s in %d\n", ptr, linecnt);
                    }
                }
            }

            if(!__issp(*ptr) && !ignore)
            {
                //printf("line %d contains invalid characters, ignored\n", linecnt);
                break;
            }

            ptr++;
        }
        ptr = str;
    }

    if(iUpperStatus != 0)
        iUpperStatus = 1;
}

